const https = require('https');

const getCedulas = (req, res) => {
  const { nombre, paterno, materno } = req.query;

  if (!nombre) {
    return res.status(400).json({ error: 'Nombre is required' });
  }

  const url = 'https://cedulaprofesional.sep.gob.mx/cedula/buscaCedulaJson.action';

  const params = new URLSearchParams({
    json: JSON.stringify({
      nombre,
      paterno,
      materno,
      idCedula: ''
    })
  });

  const options = {
    method: 'GET',
    headers: {
      'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'
    }
  };

  const request = https.get(`${url}?${params.toString()}`, options, (response) => {
    let data = '';

    response.on('data', (chunk) => {
      data += chunk;
    });

    response.on('end', () => {
      try {
        // Asegúrate de que la respuesta sea decodificada como UTF-8
        const jsonData = JSON.parse(data);
        const items = jsonData?.items || [];
        return res.status(200).json(items);
      } catch (error) {
        console.error(`Error parsing JSON: ${error.message}`);
        return res.status(500).json({ error: 'Error parsing response' });
      }
    });
  });

  request.on('error', (error) => {
    console.error(`HTTP error! Status: ${error.message}`);
    return res.status(500).json({ error: 'Internal server error' });
  });
};

module.exports = {
  getCedulas
};
