const DespachoModel = require("../models/Despachos.js");
const MateriasDespacho = require("../models/MateriasDespacho.js");
const Modulos = require("../models/Modulos.js");
const Usuarios = require("../models/Usuarios.js");
const TipoUsuarioModel = require("../models/TipoUsuarios.js");
const PaqueteModel = require("../models/Productos.js");
const Ventas = require("../models/Ventas.js");
const { sendMail } = require("../config/mail.js");
const { RegistroUsuarioHTML } = require("../Mail/RegistroUsuarioHTML.js");
const {
  sendWhatsappMovimiento,
  generatePassword,
  encriptar,
} = require("../config/functionsGlobal.js");
const EtapasProcesales = require("../models/EtapasProcesales.js");
const RecursosIncidencias = require("../models/RecursosIncidencias.js");
const RecursosDespacho = require("../models/RecursosDespacho.js");
const IncidenciasDespacho = require("../models/IncidenciasDespacho.js");
const moment = require("moment-timezone");

const APP_URL = process.env.APP_URL;

const getDespachos = async (req, res) => {
  const { estatus, page = 1, search, distribuidor } = req.query;

  const options = {
    page,
    limit: 20,
    sort: {
      _id: -1,
      nombre: 1,
    },
    populate: "estado",
    populate: "distribuidor",
  };

  const query = {};

  if (estatus) {
    query.estatus = estatus;
  }

  if (distribuidor) {
    query.$and = [...(query.$and || []), { distribuidor: distribuidor }];
  }

  if (search) {
    query.$and = [
      ...(query.$and || []),
      { nombre: { $regex: search, $options: "i" } },
    ];
  }

  try {
    console.log(query);
    const findDespachos = await DespachoModel.paginate(query, options);

    findDespachos.docs = findDespachos.docs.map((despacho) => {
      const { contadorExp } = despacho;
      const { vigencia, contador } = contadorExp;

      const diasRestantes = vigencia
        ? Math.ceil((vigencia - new Date()) / (1000 * 60 * 60 * 24))
        : 0;

      return {
        ...despacho.toJSON(),
        logo: despacho.logo
          ? `${APP_URL}/uploads/despachos/${despacho.logo}`
          : "",
        diasRestantes: diasRestantes > 0 ? diasRestantes : 0,
      };
    });

    return res.status(200).json({
      despachos: findDespachos,
    });
  } catch (error) {
    console.error("Error:", error);
    return res
      .status(500)
      .json({ message: "Error al obtener los despachos", error });
  }
};

const createDespachoRegistro = async (req, res) => {
  const { nombreCompleto, nombreDespacho, correoElectronico, telefono } =
    req.body;

  if (!nombreCompleto)
    return res.status(400).json({ message: "El nombre completo es requerido" });

  if (!nombreDespacho)
    return res
      .status(400)
      .json({ message: "El nombre del despacho es requerido" });

  if (!correoElectronico)
    return res
      .status(400)
      .json({ message: "El correo electrónico es requerido" });

  if (!telefono)
    return res.status(400).json({ message: "El teléfono es requerido" });

  // if (!estado) return res.status(400).json({ message: 'El estado es requerido' });

  // if (!municipio) return res.status(400).json({ message: 'El municipio es requerido' });

  const findUsuario = await Usuarios.findOne({ email: correoElectronico });

  if (findUsuario) {
    return res
      .status(400)
      .json({ message: "Ya existe un usuario con ese correo electrónico" });
  }

  try {
    const despachoCreate = await DespachoModel.create({
      nombre: nombreDespacho,
      razonSocial: nombreCompleto,
      correo: correoElectronico,
      telefono,
      // estado,
      // municipio,
    });

    const findModulos = await Modulos.find({
      tipo: "despacho",
      estatus: "Activo",
    }).select("_id");
    const paqueteFree = await PaqueteModel.findOne({ estatus: "Exclusivo" });

    const tipoUsuarioCreate = await TipoUsuarioModel.create({
      nombre: "ADMINISTRADOR",
      tipo: "despacho",
      despacho: despachoCreate._id,
      modulos: findModulos.map((modulo) => ({
        modulo: modulo._id,
        permisos: {
          create: true,
          read: true,
          update: true,
          delete: true,
          download: true,
        },
      })),
    });

    const password = generatePassword();
    const passwordEncripted = encriptar(password);

    const usuarioCreate = await Usuarios.create({
      nombre: nombreCompleto,
      email: correoElectronico,
      telefono,
      tipoUsuario: tipoUsuarioCreate._id,
      tipo: "despacho",
      despacho: despachoCreate._id,
      password: passwordEncripted,
    });

    // creadoPor
    await DespachoModel.findByIdAndUpdate(despachoCreate._id, {
      creadoPor: usuarioCreate._id,
    });

    const url = "login";

    sendWhatsappMovimiento({
      to: telefono,
      var1: "Gracias por registrarte en IUSFORM, tus credenciales de acceso son:",
      var2: correoElectronico,
      var3: password,
      url,
    });

    sendMail(
      RegistroUsuarioHTML(nombreCompleto, correoElectronico, password),
      "Recordatorio de contraseña",
      correoElectronico
    );

    // agregar sus materias

    const despachoId = despachoCreate._id;

    const fechaMexico = moment()
      .tz("America/Mexico_City")
      .format("YYYY-MM-DD HH:mm:ss");

    const cantidad = paqueteFree.cantidad || 10;
    const tiempo = paqueteFree.tiempo || 30;

    const fechaVigencia = moment(fechaMexico)
      .add(tiempo, "days")
      .format("YYYY-MM-DD HH:mm:ss");

    // generar la venta gratis

    Ventas.create({
      despacho: despachoCreate._id,
      usuario: usuarioCreate._id,
      producto: paqueteFree._id,
      importe: paqueteFree.precio,
      referencia: "",
      tipoReferencia: "Gratis",
      estatus: "Pagado",
    });

    await DespachoModel.findByIdAndUpdate(despachoId, {
      contadorExp: {
        contador: 0,
        limite: cantidad,
        vigencia: fechaVigencia,
      },
    });

    generalDefaults({ idDespacho: despachoId });

    const htmlRegistro = RegistroUsuarioHTML(
      nombreCompleto,
      correoElectronico,
      password,
      url
    );
    sendMail(htmlRegistro, "Bienvenido a IUSFORM", correoElectronico);

    res.status(200).json({
      despacho: despachoCreate,
      usuario: usuarioCreate,
      message:
        "Gracias por registrarte en IUSFORM, se ha enviado un correo con tus credenciales de acceso",
    });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const generalDefaults = async ({ idDespacho }) => {
  try {
    const [materias, etapasProcesales, recursos, incidencia] =
      await Promise.all([
        MateriasModel.find({ estatus: "Activo" }),
        EtapasProcesales.find({ estatus: "Activo" }),
        RecursosIncidencias.find({ estatus: "Activo", tipo: "Recurso" }),
        RecursosIncidencias.find({
          estatus: "Activo",
          tipo: "Incidencia",
        }),
      ]);

    const materiasDespachosArray = await Promise.all(
      materias.map(async (materia) => {
        const materiaDespacho = await MateriasDespacho.create({
          despacho: idDespacho,
          materia: materia._id,
          nombre: materia.nombre,
        });
        return materiaDespacho;
      })
    );

    etapasProcesales.forEach(async (etapaProcesal) => {
      const findMateria = await MateriasDespacho.findOne({
        despacho: idDespacho,
        materia: etapaProcesal.materia,
      }).select("_id");

      if (!findMateria) {
        return;
      }

      const newObjEtapa = {
        despacho: idDespacho,
        nombre: etapaProcesal.nombre,
        tipo: etapaProcesal.tipo,
        materia: findMateria._id,
      };
      await EtapasProcesales.create(newObjEtapa);
    });

    recursos.forEach(async (recurso) => {
      const findMateria = await MateriasDespacho.findOne({
        despacho: idDespacho,
        materia: recurso.materia,
      }).select("_id");

      if (!findMateria) {
        return;
      }

      const newObjRecurso = {
        despacho: idDespacho,
        nombre: recurso.nombre,
        tipo: recurso.tipo,
        materia: findMateria._id,
      };
      await RecursosDespacho.create(newObjRecurso);
    });

    incidencia.forEach(async (incidencia) => {
      const findMateria = await MateriasDespacho.findOne({
        despacho: idDespacho,
        materia: incidencia.materia,
      }).select("_id");

      if (!findMateria) {
        return;
      }

      const newObjIncidencia = {
        despacho: idDespacho,
        nombre: incidencia.nombre,
        tipo: incidencia.tipo,
        materia: findMateria._id,
      };

      await IncidenciasDespacho.create(newObjIncidencia);
    });

    return {
      materiasDespachosArray,
      etapasProcesales,
    };
  } catch (error) {
    console.log(error);
    return { error: error.message };
  }
};

const getDespachoById = async (req, res) => {
  const { id } = req.params;
  try {
    const despacho = await DespachoModel.findById(id).populate("estado");

    if (!despacho) {
      return res.status(404).json({ message: "Despacho no encontrado" });
    }

    const { contadorExp } = despacho;
    const { vigencia, contador } = contadorExp;

    const diasRestantes = vigencia
      ? Math.ceil((vigencia - new Date()) / (1000 * 60 * 60 * 24))
      : 0;

    const despachoData = {
      ...despacho.toJSON(),
      logo: despacho.logo
        ? `${APP_URL}/uploads/despachos/${despacho.logo}`
        : "",
      diasRestantes: diasRestantes > 0 ? diasRestantes : 0,
    };

    return res.status(200).json(despachoData);
  } catch (error) {
    console.error("Error:", error);
    return res
      .status(500)
      .json({ message: "Error al obtener el despacho", error });
  }
};

const updateDespacho = async (req, res) => {
  const { id } = req.params;
  const { estatus } = req.body;

  if (!estatus) {
    return res.status(400).json({ message: "Es obligatorio el estatus" });
  }
  try {
    await DespachoModel.findByIdAndUpdate(id, {
      estatus: estatus,
    });

    return res.status(200).json({ message: "Despacho actualizado" });
  } catch (error) {
    console.error("Error:", error);
    return res
      .status(500)
      .json({ message: "Error al obtener el despacho", error });
  }
};

module.exports = {
  getDespachos,
  createDespachoRegistro,
  getDespachoById,
  updateDespacho,
};
